import stringifyObject from "stringify-object";
import { getFindSelector, removeEmptyStrings } from "./utils/selector";

function getFinderString(event) {
  if (Number.isInteger(event.targetIndex)) {
    return "    cy.get('" + getFindSelector(event) + "')\n      .eq(" + event.targetIndex + ")";
  } else {
    return "    cy.get('" + getFindSelector(event) + "')";
  }
}

function getKeydownString(event) {
  switch (event.keyCode) {
    case 13:
      return getFinderString(event) + "\n    .type('{enter}')";
    default:
      return "";
  }
}

function testCommandsForFindAndSimulate(event) {
  console.log("event", event);
  switch (event.type) {
    case "keydown":
      return getKeydownString(event);
    case "input":
    case "change":
      if (event.target.type === "checkbox") {
        if (event.target.checked) {
          return getFinderString(event) + "\n      .check()";
        } else {
          return getFinderString(event) + "\n      .uncheck()";
        }
      } else {
        return getFinderString(event) + "\n      .type('" + event.target.value + "')";
      }
    case "click":
      return getFinderString(event) + ".click()";
    default:
      return "";
  }
}

export function getTestString(testName, initialProps, componentName, events) {
  var withImports = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;

  var begin = "describe('" + testName + "', () => {\n  it('should pass', () => {\n    cy.visit('/')\n";

  var findAndSimulateCommands = events.map(testCommandsForFindAndSimulate).reduce(removeEmptyStrings, [])
  //     // .reduce(squashSimilarConsecutiveEvents, [])
  .join("\n");

  var end = "\n  })\n})";

  return begin + findAndSimulateCommands + end;
}