"use strict";

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _reactModal = require("react-modal");

var _reactModal2 = _interopRequireDefault(_reactModal);

var _reactSwitch = require("react-switch");

var _reactSwitch2 = _interopRequireDefault(_reactSwitch);

var _test_viewer = require("./test_viewer");

var _enzyme_generator = require("./enzyme_generator");

var enzymeGenerator = _interopRequireWildcard(_enzyme_generator);

var _cypress_generator = require("./cypress_generator");

var cypressGenerator = _interopRequireWildcard(_cypress_generator);

var _selector = require("./utils/selector");

var _events_map = require("./events_map");

require("./base.css");

var _simple_button = require("./simple_button");

var _simple_button2 = _interopRequireDefault(_simple_button);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function eventHasTarget(event) {
  return event.target;
}

function eventOnFocussableElement(event) {
  var focusableElements = ["input"];
  return focusableElements.includes(event.target.tagName.toLowerCase());
}

function shallowDiffers(a, b) {
  for (var i in a) {
    if (!(i in b)) return true;
  }for (var _i in b) {
    if (a[_i] !== b[_i]) return true;
  }return false;
}

function capitalize(str) {
  return str[0].toUpperCase() + str.slice(1);
}

// event names are 'click', 'keyDown', 'blur' etc.
// event handlers in react are onClick, onKeyDown, onBlur
// easy to get event handler names from event names
function eventToClickHandlerMapping(eventName) {
  return "on" + capitalize(eventName);
}

/**
 * we want to listen on the Capture version of events
 * so instead of onClick, onClickCapture, since that handler is invoked in the
 * first phase of event capture - flowing from top to bottom
 * @param {string} eventName
 */
function eventToCaptureClickHandlerMapping(eventName) {
  // mouseenter and mouseleave do not have capture phase but have special bubbling
  // https://github.com/facebook/react/issues/2826
  if (eventName === "mouseEnter" || eventName === "mouseLeave") {
    return eventToClickHandlerMapping(eventName);
  } else {
    return eventToClickHandlerMapping(eventName) + "Capture";
  }
}

var customEvents = {
  COMPONENT_WILL_RECEIVE_PROPS: "COMPONENT_WILL_RECEIVE_PROPS"

  /**
   * events which we throw - like throwing an event for componentWillReceiveProp lifecycle method being called
   * @param {Object} event
   */
};function isCustomEvent(event) {
  return Object.values(customEvents).includes(event.type);
}

/**
 * we take properties from the event which might be needed to be passed on while simulating the event
 * Or helpful in any other way while creating the test string
 * @param {Object} event
 */
function createEventObject(eventName, event) {
  var id = event.target && event.target.getAttribute("id");
  var dataTestId = event.target && event.target.getAttribute("data-test-id");

  var ourEvent = _extends({
    name: eventName,
    type: event.type,
    key: event.key,
    keyCode: event.keyCode,
    which: event.which
  }, _events_map.allEvents[eventName].reduce(function (acc, propName) {
    var _extends2;

    return _extends({}, acc, (_extends2 = {}, _extends2[propName] = event[propName], _extends2));
  }, {}), {
    target: {
      "data-test-id": dataTestId,
      id: id,
      tagName: event.target.tagName,
      value: event.target.value,
      checked: event.target.checked,
      type: event.target && event.target.type,
      getAttribute: function getAttribute(attr) {
        if (attr === "id") {
          return id;
        } else if (attr === "data-test-id") {
          return dataTestId;
        } else {
          return null;
        }
      }
    }
  });

  function moreThanOnePossibleTargets(event) {
    return document.querySelectorAll((0, _selector.getFindSelector)(event)).length > 1;
  }

  function findIndexOfTargetInPossibleCandidates(event) {
    var i = -1;

    document.querySelectorAll((0, _selector.getFindSelector)(event)).forEach(function (el, index) {
      if (el === event.target) {
        i = index;
      }
    });

    return i;
  }

  if (!isCustomEvent(event) && moreThanOnePossibleTargets(event)) {
    // if we find more than one element matching the selection criteria we defined,
    // let's find the index of that element
    return _extends({}, ourEvent, {
      targetIndex: findIndexOfTargetInPossibleCandidates(event)
    });
  } else {
    return ourEvent;
  }
}

function testGenerator(Component) {
  return function (_React$Component) {
    _inherits(TestGeneratorComponent, _React$Component);

    function TestGeneratorComponent(props) {
      _classCallCheck(this, TestGeneratorComponent);

      var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

      _this.events = [];
      _this.state = {
        generatedTest: "",
        errorHappened: false,
        testName: "Interaction test 1",
        showExcludeModal: false,
        excludedEvents: [],
        excludeEventSearchString: "",
        excludeEventsSearchFilter: "all",
        generator: "enzyme",
        withImports: false
      };
      _this.initialProps = null;
      _this.eventWrapperRef = null;

      _this.componentDidCatch = function (error, info) {
        console.log("error somewhere in the component tree");
        _this.setState({ errorHappened: true, testName: "Breaking test" });
      };

      _this.startTestGenertion = function () {
        _this.events = [];
      };

      _this.stopTestGenertion = function () {
        _this.setState({
          generatedTest: _this.getTestString()
        });
      };

      _this.recordEvent = function (eventName, event) {
        function targetListeningOnEvent(eventName, event, eventWrapperRef) {
          // the event object has a weird property named something like '__reactEventHandlers$<random_chars>
          // which contains info about the input as setup in jsx
          // so it includes the event handlers setup in jsx by the user - like onKeyDown, onClick etc.
          var reactEventObjectProp = Object.keys(event.target).filter(function (key) {
            return key.includes("__reactEventHandlers");
          })[0];

          if (reactEventObjectProp) {
            // sometimes user might have setup click handling (or any other handling)
            // on a parent and the actual event happens on a leaf element
            // in that case what we get as event.target is not what the user has attached the handler on
            // we were filtering those events, even though they are important
            // need a more robust way of filtering those events
            // what about checking if the element where the event occured is inside the Component we are testing
            // that provides some filtering and hence narrowing but still rules out extraneous events
            return eventWrapperRef && eventWrapperRef.contains(event.target); // just checks if the event happened on something inside the component we are testing
            // return (
            //   typeof event.target[reactEventObjectProp][
            //     eventToClickHandlerMapping(eventName)
            //   ] === "function"
            // )
          } else {
            // we make one exception for events happening on document body
            // those are the ones created using document.addEventListener
            // very common for keyboard events
            return event.target === document.body || false;
          }
        }

        function targetNotEventWrapper(event, eventWrapperRef) {
          return event.target !== eventWrapperRef;
        }

        // only record event if the target element is also listening on the same event
        if ((targetListeningOnEvent(eventName, event, _this.eventWrapperRef) || isCustomEvent(event)) && targetNotEventWrapper(event, _this.eventWrapperRef)) {
          _this.events.push(createEventObject(eventName, event));
        } else {
          // console.log("event rejected", eventName)
        }
      };

      _this.getTestString = function () {
        var withImports = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
        var _this$state = _this.state,
            testName = _this$state.testName,
            excludedEvents = _this$state.excludedEvents;

        if (_this.state.generator === "cypress") {
          return cypressGenerator.getTestString(testName, _this.initialProps, Component.name, _this.events.filter(function (event) {
            return !excludedEvents.includes(event.name);
          }), withImports);
        } else {
          return enzymeGenerator.getTestString(testName, _this.initialProps, Component.name, _this.events.filter(function (event) {
            return !excludedEvents.includes(event.name);
          }), withImports);
        }
      };

      _this.handleTestNameChange = function (event) {
        _this.setState({ testName: event.target.value });
      };

      _this.handleExcludeClick = function () {
        _this.setState({ showExcludeModal: true });
      };

      _this.saveExcludedItemsToLocalStorage = function () {
        localStorage.setItem("excluded_events", JSON.stringify(_this.state.excludedEvents));
      };

      _this.handleEventExcludeChange = function (eventName, checked) {
        var excludedEvents = _this.state.excludedEvents;


        if (!checked) {
          _this.setState({
            excludedEvents: excludedEvents.concat(eventName)
          }, _this.saveExcludedItemsToLocalStorage);
        } else {
          var index = excludedEvents.indexOf(eventName);

          if (index >= 0) {
            _this.setState({
              excludedEvents: [].concat(excludedEvents.slice(0, index), excludedEvents.slice(index + 1))
            }, _this.saveExcludedItemsToLocalStorage);
          }
        }
      };

      _this.handleGeneratorChange = function (event) {
        _this.setState({
          generator: event.target.value
        });
      };

      _this.handleWithImportsChange = function (event) {
        _this.setState({
          withImports: event.target.checked,
          generatedTest: _this.getTestString(event.target.checked)
        });
      };

      _this.initialProps = _extends({}, props);
      try {
        var excludedEvents = JSON.parse(localStorage.getItem("excluded_events"));
        _this.state.excludedEvents = excludedEvents || [];
      } catch (e) {
        console.error("Error gettting excluded events from local storage", e);
      }
      return _this;
    }

    TestGeneratorComponent.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
      if (shallowDiffers(this.props, nextProps)) {
        this.events.push({
          eventFromTestGenerator: true,
          type: customEvents.COMPONENT_WILL_RECEIVE_PROPS,
          nextProps: nextProps
        });
      }
    };

    TestGeneratorComponent.prototype.componentDidMount = function componentDidMount() {
      // document.addEventListener(
      //   "keydown",
      //   this.recordEvent.bind(this, "keyDown")
      // )
    };

    TestGeneratorComponent.prototype.componentWillUnmount = function componentWillUnmount() {
      // document.removeEventListener(
      //   "keydown",
      //   this.recordEvent.bind(this, "keyDown")
      // )
    };

    TestGeneratorComponent.prototype.getEventHandlers = function getEventHandlers() {
      var _this2 = this;

      var eventHandlers = _events_map.allEventNames.reduce(function (acc, eventName) {
        var _extends3;

        return _extends({}, acc, (_extends3 = {}, _extends3[eventToCaptureClickHandlerMapping(eventName)] = _this2.recordEvent.bind(_this2, eventName), _extends3));
      }, {});

      return eventHandlers;
    };

    TestGeneratorComponent.prototype.render = function render() {
      var _this3 = this;

      var _state = this.state,
          errorHappened = _state.errorHappened,
          excludeEventsSearchFilter = _state.excludeEventsSearchFilter,
          excludedEvents = _state.excludedEvents,
          excludeEventSearchString = _state.excludeEventSearchString,
          testName = _state.testName,
          showExcludeModal = _state.showExcludeModal,
          generatedTest = _state.generatedTest,
          generator = _state.generator,
          withImports = _state.withImports;


      if (errorHappened) {
        return _react2.default.createElement(
          "div",
          { style: { marginTop: 10 } },
          _react2.default.createElement(
            "h3",
            { style: { marginBottom: 20 } },
            "Looks like your app crashed \uD83D\uDCA3\uD83D\uDCA5. This might help \uD83D\uDC40."
          ),
          _react2.default.createElement(_test_viewer.TestViewer, {
            testString: this.getTestString(this.state.withImports),
            onWithImportsChange: this.handleWithImportsChange,
            withImports: withImports
          })
        );
      }

      return _react2.default.createElement(
        "div",
        null,
        _react2.default.createElement(
          "div",
          _extends({}, this.getEventHandlers(), {
            ref: function ref(node) {
              return _this3.eventWrapperRef = node;
            }
          }),
          _react2.default.createElement(Component, this.props)
        ),
        _react2.default.createElement(
          "div",
          { style: { marginTop: 20 } },
          _react2.default.createElement(_test_viewer.TestViewerContainer, {
            componentName: Component.name,
            testString: generatedTest,
            onGetTestClick: this.stopTestGenertion,
            onStartTestGeneration: this.startTestGenertion,
            onStopTestGeneration: this.stopTestGenertion,
            testName: testName,
            onTestNameChange: this.handleTestNameChange,
            onExcludeClick: this.handleExcludeClick,
            excludedEvents: excludedEvents,
            onGeneratorChange: this.handleGeneratorChange,
            generator: generator,
            withImports: withImports,
            onWithImportsChange: this.handleWithImportsChange
          })
        ),
        _react2.default.createElement(
          _reactModal2.default,
          {
            isOpen: showExcludeModal,
            onRequestClose: function onRequestClose() {
              return _this3.setState({
                showExcludeModal: false,
                excludeEventSearchString: "",
                generatedTest: _this3.getTestString(withImports)
              });
            },
            style: {
              overlay: {},
              content: {
                width: "50%",
                margin: "auto",
                padding: 30
              }
            }
          },
          _react2.default.createElement(
            "div",
            {
              style: {
                display: "flex",
                justifyContent: "space-between"
              }
            },
            _react2.default.createElement(
              "h3",
              { style: { maxWidth: "80%" } },
              "Toggle switch to exclude an event. Green means it's included."
            ),
            _react2.default.createElement(
              _simple_button2.default,
              {
                onClick: function onClick() {
                  return _this3.setState({
                    showExcludeModal: false,
                    excludeEventSearchString: "",
                    generatedTest: _this3.getTestString(withImports)
                  });
                }
              },
              "close"
            )
          ),
          _react2.default.createElement(
            "div",
            {
              style: {
                display: "flex",
                justifyContent: "space-between",
                alignItems: "center",
                marginTop: 10
              }
            },
            _react2.default.createElement(
              "div",
              null,
              _react2.default.createElement("input", {
                style: {
                  padding: "0.3em 0.6em",
                  marginBottom: 5
                },
                onChange: function onChange(e) {
                  return _this3.setState({ excludeEventSearchString: e.target.value });
                },
                value: excludeEventSearchString,
                autoFocus: true
              })
            ),
            _react2.default.createElement(
              "div",
              {
                style: { display: "flex", alignItems: "center" },
                "data-test-id": "src_footer_test_id_2"
              },
              _react2.default.createElement(
                "div",
                { style: { marginRight: 10 } },
                excludedEvents.length,
                " events excluded"
              ),
              _react2.default.createElement(
                "a",
                {
                  onClick: function onClick() {
                    return _this3.setState({ excludeEventsSearchFilter: "all" });
                  },
                  style: {
                    margin: 3,
                    padding: "3px 7px",
                    textDecoration: "none",
                    border: excludeEventsSearchFilter === "all" ? "1px solid black" : "none",
                    borderRadius: 3,
                    cursor: "pointer"
                  }
                },
                "All"
              ),
              _react2.default.createElement(
                "a",
                {
                  onClick: function onClick() {
                    return _this3.setState({ excludeEventsSearchFilter: "excluded" });
                  },
                  style: {
                    margin: 3,
                    padding: "3px 7px",
                    textDecoration: "none",
                    border: excludeEventsSearchFilter === "excluded" ? "1px solid black" : "none",
                    borderRadius: 3,
                    cursor: "pointer"
                  },
                  "data-test-id": "src_footer_test_id_4"
                },
                "Excluded"
              )
            )
          ),
          _react2.default.createElement(
            "ul",
            { style: { padding: 0 } },
            _events_map.allEventNames.filter(function (eventName) {
              return eventName.toLowerCase().includes(excludeEventSearchString.toLocaleLowerCase());
            }).filter(function (eventName) {
              return excludeEventsSearchFilter === "all" || excludedEvents.includes(eventName);
            }).map(function (eventName, index) {
              return _react2.default.createElement(
                "li",
                {
                  style: {
                    display: "flex",
                    flexDirection: "row",
                    alignItems: "center",
                    padding: "0.5em 0.6em",
                    borderBottom: "1px solid #eee"
                  },
                  key: "exclude_list_item_" + index
                },
                _react2.default.createElement(_reactSwitch2.default, {
                  onChange: _this3.handleEventExcludeChange.bind(_this3, eventName),
                  checked: !excludedEvents.includes(eventName),
                  id: "switch_id_" + eventName
                }),
                _react2.default.createElement(
                  "div",
                  { style: { marginLeft: 20, fontWeight: 700 } },
                  eventName
                )
              );
            })
          )
        )
      );
    };

    return TestGeneratorComponent;
  }(_react2.default.Component);
}

exports.default = testGenerator;
module.exports = exports["default"];